/***
 *** PCI16AI64_IOCTL.H 
 ***
 ***  General description of this file:
 ***  	IOCTL code and structure definitions for General Standards PCI-16AI64 
 ***  	16 channel PCI A/D board. This file is part of the Linux
 ***  	driver distribution for this board.
 ***  	
 ***  	This file is necessary to compile application programs, therefore 
 ***  	should be included in binary only driver distributions.
 ***
 ***	In the following, 'local' registers are board-specific, as
 ***	contrasted to the registers located on the PLX PCI bus
 ***	access chip.
 ***
 ***  Copyrights (c):
 ***  	General Standards Corporation (GSC), Nov 2002
 ***
 ***  Author:
 ***  	Evan Hillman, GSC Inc. (evan@generalstandards.com)
 ***
 ***  Support:
 ***  	Primary support for this driver is provided by GSC. 
 ***
 ***  Platform (tested on, may work with others):
 ***  	Linux, kernel version 2.2.x, Intel hardware.
 ***
 ***  Revision history:
 ***    Nov 2002: initial release
 ***/

#define DEFAULT_TIMEOUT 10
#define DEFAULT_TIMEOUT_MSEC MSECS_TO_SLEEP(DEFAULT_TIMEOUT*1000)

 
#ifndef __DEVICE_IOCTL_H_INCLUDED__
#define __DEVICE_IOCTL_H_INCLUDED__

#define IOCTL_DEVICE_MAGIC   's'

 /**
  ** IOCTL defines.  See below for API details.
  **
  **/

#define IOCTL_DEVICE_NO_COMMAND                _IO  (IOCTL_DEVICE_MAGIC,1)
#define IOCTL_DEVICE_INIT_BOARD                _IO  (IOCTL_DEVICE_MAGIC,2)
#define IOCTL_DEVICE_READ_REGISTER             _IOWR(IOCTL_DEVICE_MAGIC,3, struct device_register_params)  
#define IOCTL_DEVICE_WRITE_REGISTER            _IOWR(IOCTL_DEVICE_MAGIC,4, struct device_register_params)   
#define IOCTL_DEVICE_READ_PCI_CONFIG           _IOWR(IOCTL_DEVICE_MAGIC,5, struct device_register_params)
#define IOCTL_DEVICE_WRITE_PCI_CONFIG          _IOWR(IOCTL_DEVICE_MAGIC,6, struct device_register_params)
#define IOCTL_DEVICE_ENABLE_DMA                _IOW (IOCTL_DEVICE_MAGIC,7, unsigned long)
#define IOCTL_DEVICE_REG_FOR_INT_NOTIFY        _IOWR(IOCTL_DEVICE_MAGIC,8, struct device_int_notify_params)
#define IOCTL_DEVICE_GET_DEVICE_ERROR          _IOR (IOCTL_DEVICE_MAGIC,9, unsigned long)
#define IOCTL_DEVICE_INPUT_RANGE_CONFIG        _IOW (IOCTL_DEVICE_MAGIC,10,unsigned long)
#define IOCTL_DEVICE_INPUT_MODE_CONFIG         _IOW (IOCTL_DEVICE_MAGIC,11,unsigned long)
#define IOCTL_DEVICE_INPUT_FORMAT_CONFIG       _IOW (IOCTL_DEVICE_MAGIC,12,unsigned long)
#define IOCTL_DEVICE_START_AUTOCAL			   _IO  (IOCTL_DEVICE_MAGIC,13)
#define IOCTL_DEVICE_START_SINGLE_SCAN         _IO  (IOCTL_DEVICE_MAGIC,14)
#define IOCTL_DEVICE_SET_SCAN_SIZE             _IOW (IOCTL_DEVICE_MAGIC,15,unsigned long)
#define IOCTL_DEVICE_SET_SCAN_CLOCK            _IOW (IOCTL_DEVICE_MAGIC,16,unsigned long)
#define IOCTL_DEVICE_DISABLE_PCI_INTERRUPTS    _IO  (IOCTL_DEVICE_MAGIC,17)
#define IOCTL_DEVICE_SET_B_CLOCK_SOURCE        _IOW (IOCTL_DEVICE_MAGIC,18,unsigned long)
#define IOCTL_DEVICE_SINGLE_CHAN_SELECT        _IOW (IOCTL_DEVICE_MAGIC,19,unsigned long)
#define IOCTL_DEVICE_SET_SCAN_RATE_A           _IOR (IOCTL_DEVICE_MAGIC,20,unsigned long)
#define IOCTL_DEVICE_SET_SCAN_RATE_B           _IOW (IOCTL_DEVICE_MAGIC,21,unsigned long)
#define IOCTL_DEVICE_GEN_DISABLE_A             _IO  (IOCTL_DEVICE_MAGIC,22)
#define IOCTL_DEVICE_GEN_DISABLE_B             _IO  (IOCTL_DEVICE_MAGIC,23)
#define IOCTL_DEVICE_GEN_ENABLE_A              _IO  (IOCTL_DEVICE_MAGIC,24)
#define IOCTL_DEVICE_GEN_ENABLE_B              _IO  (IOCTL_DEVICE_MAGIC,25)
#define IOCTL_DEVICE_SET_BUFFER_THRESHOLD      _IOW (IOCTL_DEVICE_MAGIC,26,unsigned long)
#define IOCTL_DEVICE_CLEAR_BUFFER              _IO  (IOCTL_DEVICE_MAGIC,27)
#define IOCTL_DEVICE_GET_DEVICE_TYPE           _IOR (IOCTL_DEVICE_MAGIC,28,unsigned long)

/*****************************************************************************
 ** IOCTL_DEVICE_NO_COMMAND
 **
 ** Parameter = NONE
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_INIT_BOARD
 **
 ** Parameter = NONE
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_READ_REGISTER	  
 ** IOCTL_DEVICE_WRITE_REGISTER	  
 ** IOCTL_DEVICE_WRITE_PCI_CONFIG_REG
 ** IOCTL_DEVICE_WRITE_LOCAL_CONFIG_REG
 **
 ** Parameter = DEVICE_REGISTER_PARAMS *pRegParams;
 **
 ** This structure is used to store information about a register. The
 ** IOCTL_DEVICE_READ_REGISTER and IOCTL_DEVICE_WRITE_REGISTER ioctl commands use this
 ** structure to read a particular register and to write a value into a particular
 ** register. 'DeviceRegister' stores the index of the register, range being 0-94, and
 ** 'ulRegisterValue' stores the register value, range being 0x0-0xFFFFFFFF.
 **/
typedef struct device_register_params {
  unsigned int DeviceRegister;	      	    /* range: 0-94, see definitions below */
  unsigned long ulRegisterValue;	    /* range: 0x0-0xFFFFFFFF, same register values below */
} DEVICE_REGISTER_PARAMS, *PDEVICE_REGISTER_PARAMS;


/* ==== Register Offsets ==== */
/* ---- 'local' Registers ---- */

#define BOARD_CTRL_REG			0
#define INT_CTRL_REG			1
#define INPUT_DATA_BUFF_REG		2
#define INPUT_BUFF_CTRL_REG		3
#define RATE_A_GENERATOR_REG	4
#define RATE_B_GENERATOR_REG	5
#define BUFFER_SIZE_REG			6
#define RESERVED_1_REG			7
#define SCAN_SYNCH_CTRL_REG		8


/* ---- DMA Registers ---- */
/* ---- see PLX documentation for info ---- */

#define DMA_CH_0_MODE                	    32
#define DMA_CH_0_PCI_ADDR            	    33
#define DMA_CH_0_LOCAL_ADDR          	    34
#define DMA_CH_0_TRANS_BYTE_CNT      	    35
#define DMA_CH_0_DESC_PTR            	    36
#define DMA_CH_1_MODE                	    37
#define DMA_CH_1_PCI_ADDR            	    38
#define DMA_CH_1_LOCAL_ADDR          	    39
#define DMA_CH_1_TRANS_BYTE_CNT      	    40
#define DMA_CH_1_DESC_PTR            	    41
#define DMA_CMD_STATUS               	    42
#define DMA_MODE_ARB_REG             	    43
#define DMA_THRESHOLD_REG            	    44

/* ---- PCI Configuration Registers ---- */
/* ---- see PLX documentation for info ---- */

#define DEVICE_VENDOR_ID              	    0
#define STATUS_COMMAND                      1
#define CLASS_CODE_REVISION_ID              2
#define BIST_HDR_TYPE_LAT_CACHE_SIZE        3
#define PCI_MEM_BASE_ADDR                   4
#define PCI_IO_BASE_ADDR                    5
#define PCI_BASE_ADDR_0                     6
#define PCI_BASE_ADDR_1                     7
#define CARDBUS_CIS_PTR                     10
#define SUBSYS_ID_VENDOR_ID                 11
#define PCI_BASE_ADDR_LOC_ROM               12
#define LAT_GNT_INT_PIN_LINE                15

/* ---- Local Configuration Registers ---- */
/* ---- see PLX documentation for info ---- */

#define PCI_TO_LOC_ADDR_0_RNG               0
#define LOC_BASE_ADDR_REMAP_0               1
#define MODE_ARBITRATION                    2
#define BIG_LITTLE_ENDIAN_DESC              3
#define PCI_TO_LOC_ROM_RNG                  4
#define LOC_BASE_ADDR_REMAP_EXP_ROM         5
#define BUS_REG_DESC_0_FOR_PCI_LOC          6
#define DIR_MASTER_TO_PCI_RNG               7
#define LOC_ADDR_FOR_DIR_MASTER_MEM         8
#define LOC_ADDR_FOR_DIR_MASTER_IO          9
#define PCI_ADDR_REMAP_DIR_MASTER           10
#define PCI_CFG_ADDR_DIR_MASTER_IO          11
#define PCI_TO_LOC_ADDR_1_RNG               92
#define LOC_BASE_ADDR_REMAP_1               93
#define BUS_REG_DESC_1_FOR_PCI_LOC          94

/* ---- Run Time Registers ---- */
/* ---- see PLX documentation for info ---- */

#define MAILBOX_REGISTER_0                  16
#define MAILBOX_REGISTER_1                  17
#define MAILBOX_REGISTER_2                  18
#define MAILBOX_REGISTER_3                  19
#define MAILBOX_REGISTER_4                  20
#define MAILBOX_REGISTER_5                  21
#define MAILBOX_REGISTER_6                  22
#define MAILBOX_REGISTER_7                  23
#define PCI_TO_LOC_DOORBELL                 24
#define LOC_TO_PCI_DOORBELL                 25
#define INT_CTRL_STATUS                     26
#define PROM_CTRL_CMD_CODES_CTRL            27
#define DEVICE_ID_VENDOR_ID                 28
#define REVISION_ID                         29
#define MAILBOX_REG_0                       30
#define MAILBOX_REG_1                       31

/* ---- Messaging Queue Registers ---- */
/* ---- see PLX documentation for info ---- */

#define OUT_POST_Q_INT_STATUS               12
#define OUT_POST_Q_INT_MASK                 13
#define IN_Q_PORT                           16
#define OUT_Q_PORT                          17
#define MSG_UNIT_CONFIG                     48
#define Q_BASE_ADDR                         49
#define IN_FREE_HEAD_PTR                    50
#define IN_FREE_TAIL_PTR                    51
#define IN_POST_HEAD_PTR                    52
#define IN_POST_TAIL_PTR                    53
#define OUT_FREE_HEAD_PTR                   54
#define OUT_FREE_TAIL_PTR                   55
#define OUT_POST_HEAD_PTR                   56
#define OUT_POST_TAIL_PTR                   57
#define Q_STATUS_CTRL_REG                   58

/* ==== Values for some key registers ==== */

/* ---- masks for the 'local' Board Control Register (32-bit addr 0). ---- */
/* ---- see GSC documentation for info ---- */

// analog input mode

#define BC_DIFFERENTIAL 0
#define BC_SINGLE_ENDED 1
#define BC_ZERO_TEST 2
#define BC_VREF_TEST 3

#define BC_AIM_MASK                         0x0007
#define BC_AIM_SHIFT                        0

// analog input range

#define BC_RANGE_SHIFT                      4

#define BC_RANGE_2_5						(0)
#define BC_RANGE_5	  					    (1)
#define BC_RANGE_10 						(2)

#define BC_RANGE_MASK                       (3<<BC_RANGE_SHIFT)

// offset binary mode 

#define BC_FORMAT_MASK                      (1<<6)
#define BC_OFFSET_BINARY_SHIFT              6

// External synch

#define BC_ENABLE_EXTERN_SYNC               (1<<7)

// start single scan

#define BC_INPUT_SYNCH_START				(1<<12)

// init autocalibration

#define BC_AUTOCAL                          (1<<13)

// autocalibration test passed flag

#define BC_AUTOCAL_PASS                     (1<<14)

// initialize the hardware

#define BC_INITIALIZE                       (1<<15)

/* ---- masks for the 'local' Interrupt Control Register (32-bit addr 1). ---- */
/* ---- see GSC documentation for info ---- */

#define IRQ_0_IDLE 0
#define IRQ_0_AUTOCAL_COMPLETE 1
#define IRQ_0_INPUT_SCAN_INIT 2
#define IRQ_0_INPUT_SCAN_COMPLETE 3
#define IRQ_0_MASK 0x07

#define IRQ_0_REQUEST (1<<3)

#define IRQ_1_IDLE (0)
#define IRQ_1_LO_HI (1<<4)
#define IRQ_1_HI_LO (2<<4)
#define IRQ_1_MASK (0x03<<4)

#define IRQ_1_REQUEST (1<<7)

/* ---- masks for the 'local' Input Data Register (32-bit addr 2). ---- */
/* ---- see GSC documentation for info ---- */

#define VALID_DATA_MASK 0XFF

/* ---- masks for the 'local' Input Data Control Registers (32-bit addr 3). ---- */
/* ---- see GSC documentation for info ---- */

#define IDC_INPUT_CLEAR_BUFFER (1<<16)
#define IDC_THRESHOLD_FLAG (1<<17)
#define IDC_THRESHOLD_MASK 0xFFFF

/* ---- masks for the 'local' Rate A Generator Register (32-bit addr 4). ---- */
/* ---- masks for the 'local' Rate B Generator Register (32-bit addr 5). ---- */
/* ---- see GSC documentation for info ---- */

#define RGR_NRATE_MASK (0xFFFF)
#define RGR_GEN_DISABLE (1<<16)

/* ---- masks for the 'local' Buffer Size (32-bit addr 6). ---- */
/* ---- see GSC documentation for info ---- */

/* no masks required... */

/* ---- masks for the 'local' Scan and Synch Control Register(32-bit addr 8). ---- */
/* ---- see GSC documentation for info ---- */

// Scan size defines

#define SSCR_1_CHAN_SIZE 0
#define SSCR_2_CHAN_SIZE 1
#define SSCR_4_CHAN_SIZE 2
#define SSCR_8_CHAN_SIZE 3
#define SSCR_16_CHAN_SIZE 4
#define SSCR_32_CHAN_SIZE 5
#define SSCR_64_CHAN_SIZE 6

#define SSCR_CHAN_SIZE_MASK 0x03

// analog inputs scan clock

#define SSCR_CLOCK_SHIFT 3
#define SSCR_SCAN_CLOCK_MASK (3<<SSCR_CLOCK_SHIFT)

#define SSCR_INTERNAL_RATE_A 0
#define SSCR_INTERNAL_RATE_B (1)
#define SSCR_EXTERNAL_SYNCH  (2)
#define SSCR_BCR_INPUT_SYNCH (3)

// single channel defines

#define SSCR_SINGLE_CHANNEL_SHIFT (12)
#define SSCR_SINGLE_CHANNEL_MASK (0x3F<<12)

// Rate "B" clock source

#define SSCR_RATE_B_CLOCK_SOURCE_SHIFT (10)
#define SSCR_RATE_B_CLOCK_SOURCE_MASK (1<<SSCR_RATE_B_CLOCK_SOURCE_SHIFT)

#define SSCR_MASTER_CLOCK 0
#define SSCR_RATE_A_GENERATOR (1)

/* ---- DMA command/status register bits for channel 0 ---- */
/* ---- see PLX documentation for info ---- */

#define DMA0_ENABLE			    0x0001
#define DMA0_START			    0x0002
#define DMA0_ABORT			    0x0004
#define DMA0_CLR_INT		    0x0008
#define DMA0_DONE			    0x0010

/*****************************************************************************
 ** IOCTL_DEVICE_ENABLE_DMA
 **
 ** Parameter = DMA_PARAMS *DMAParameters;
 **
 ** Set up to use DMA for data transfer.
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_REQ_INT_NOTIFY
 **
 ** Parameter = DEVICE_INT_NOTIFY_PARAMS *pIntNotify;
 **
 ** This structure stores information about an event to be signaled at the time
 ** of an interrupt. The IOCTL_DEVICE_REQ_INT_NOTIFY ioctl command uses this structure
 ** to notify a particular event that an interrupt occurred and to pass on the condition.
 ** IntCond contains a condition to be notified, range being 0-6, and Event stores the
 ** event to be signaled when the interrupt occurs.
 **/
typedef struct device_int_notify_params {
  unsigned int eIntCondition;	      	    /* see condition codes below */
  int iSignal;	      	      	      	    /* signal number (turn off if less than 0) */
} DEVICE_INT_NOTIFY_PARAMS, *PDEVICE_INT_NOTIFY_PARAMS;

/* ---- event condition codes ---- */
#define EVENT_IDLE                      0
#define EVENT_AUTOCAL_COMPLETE          1
#define EVENT_SCAN_STARTED              2
#define EVENT_SCAN_COMPLETED            3
#define EVENT_BUFFER_HIGH_TO_LOW        4
#define EVENT_BUFFER_LOW_TO_HIGH        5

/*****************************************************************************
 ** IOCTL_DEVICE_GET_DEVICE_ERROR
 **
 ** Parameter = unsigned long *pulDeviceError;
 ** Return value range: 0-8, see codes below
 **/

#define DEVICE_SUCCESS                         0
#define DEVICE_INVALID_PARAMETER               1
#define DEVICE_INVALID_BUFFER_SIZE             2
#define DEVICE_PIO_TIMEOUT                     3
#define DEVICE_DMA_TIMEOUT                     4
#define DEVICE_IOCTL_TIMEOUT                   5
#define DEVICE_OPERATION_CANCELLED             6
#define DEVICE_RESOURCE_ALLOCATION_ERROR       7
#define DEVICE_INVALID_REQUEST                 8
#define DEVICE_AUTOCAL_FAILED    	      	   9

/*****************************************************************************
 ** IOCTL_DEVICE_INPUT_RANGE_CONFIG
 **
 ** Parameter = unsigned long *pInputRange;
 ** Range: 0-2, see codes below
 **/

/* ---- scan mode codes ---- */
#define INPUT_RANGE_2_5 0
#define INPUT_RANGE_5   1
#define INPUT_RANGE_10  3

/*****************************************************************************
 **  IOCTL_DEVICE_INPUT_MODE_CONFIG
 **
 **  Parameter = unsigned long *iMode
 **/

/* ---- mode config codes ---- */
#define INPUT_MODE_SHIFT	0
#define INPUT_MODE_MASK		0x07

#define INPUT_DIFFERENTIAL  0
#define INPUT_SINGLE_ENDED  1
#define INPUT_ZERO_TEST     2
#define INPUT_VREF_TEST     3

/*****************************************************************************
 **  IOCTL_DEVICE_INPUT_FORMAT_CONFIG
 **
 **  Parameter = unsigned long *format
 ** Range: 0-3, see codes below
 **/

/* ---- device test codes ---- */
#define FORMAT_TWO_COMPLEMENT     0
#define FORMAT_OFFSET_BINARY      1

/*****************************************************************************
 **  IOCTL_DEVICE_START_AUTOCAL
 **
 **  Parameter =  NONE
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_START_SINGLE_SCAN
 **
 ** Parameter = NONE;
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_SET_SCAN_SIZE
 **
 ** Parameter = unsigned long *pScanSize;
 ** Range: 0-6, see codes below
 **/

/* ---- scan size codes ---- */
#define SCAN_SINGLE_CHAN	0
#define SCAN_2_CHAN         1
#define SCAN_4_CHAN		    2
#define SCAN_8_CHAN		    3
#define SCAN_16_CHAN		4
#define SCAN_32_CHAN		5
#define SCAN_64_CHAN		6

/*****************************************************************************
 ** IOCTL_DEVICE_SET_SCAN_CLOCK
 **
 ** select the analog input scan clock
 **
 ** Parameter = unsigned long *clock
 ** range=0-3
 **/

#define INTERNAL_RATE_A	0
#define INTERNAL_RATE_B	1
#define EXTERNAL_SYNCH	2
#define BCR_INPUT_SYNCH	3

/*****************************************************************************
 **  IOCTL_DEVICE_DISABLE_PCI_INTERRUPTS
 **
 **  Parameter = NONE
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_SET_B_CLOCK_SOURCE
 **
 ** Parameter = unsigned long *pSource;
 ** Range: 0-1, see codes below
 **/

#define MASTER_CLOCK     0
#define RATE_A_OUTPUT    1

/*****************************************************************************
 ** IOCTL_DEVICE_SINGLE_CHAN_SELECT
 **
 ** Parameter =  unsigned long *channel;
 ** Range: 0-64
 **
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_SET_SCAN_RATE_A
 **
 ** Parameter = unsigned long *setting;
 ** Range = 0-0xFFFF
 **
 **/

/*****************************************************************************
 ** IOCTL_DEVICE_SET_SCAN_RATE_B
 **
 ** Parameter = unsigned long *setting;
 ** Range = 0-0xFFFF
 **
 **/

/*****************************************************************************
**  IOCTL_DEVICE_GEN_DISABLE_A
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_GEN_DISABLE_B
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_GEN_ENABLE_A
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_GEN_ENABLE_B
**
**  Parameter = NONE
**/

/*****************************************************************************
**  IOCTL_DEVICE_SET_BUFFER_THRESHOLD
**
**  Parameter = unsigned long * threshold
**  Range = 0..0xFFFF
**/

/*****************************************************************************
**  IOCTL_DEVICE_CLEAR_BUFFER
**
**  Parameter = NONE
**/

#endif	/* entire file */

