 #include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>

#ifndef TRUE
#define TRUE (1==1)
#endif

#ifndef FALSE
#define FALSE (!TRUE)
#endif

#include "gsc16AI64_ioctl.h"
#include "kbhit.h"

#define MAX_CHANNELS 64

#define BUFFER_SIZE (1024*16/*17*//*256*/) // bytes, rather arbitrary
#define BUFFER_SIZE_WORDS (BUFFER_SIZE * sizeof(unsigned long))

char driver_name[80]="/dev/gsc16ai64\0";

char *errorString[]=
{
"DEVICE_SUCCESS",
"DEVICE_INVALID_PARAMETER",
"DEVICE_INVALID_BUFFER_SIZE",
"DEVICE_PIO_TIMEOUT",
"DEVICE_DMA_TIMEOUT",
"DEVICE_IOCTL_TIMEOUT",
"DEVICE_OPERATION_CANCELLED",
"DEVICE_RESOURCE_ALLOCATION_ERROR",
"DEVICE_INVALID_REQUEST",
"DEVICE_AUTOCAL_FAILED",
"DEVICE_BUFFER_OVERFLOW",
};

/* text names to describe each type of board supported. */
char *boards_supported[] = {
	"PMC-12AI64",       /* PMC-12AI64 */
	"PMC-16AI64",	    /* PMC-16AI64 */
	"PMC-16AI64SS/SSA",	/* PMC-16AI64SS or SSA board */
	"null"          /* null terminated list.... */
};


// hardware buffer size.  These are the factory defaults, 
// and custom hardware may be different. 
int hwBufferSize[]= {
    64*1024, /* PMC-12AI64 */
    64*1024,/* PMC-16AI64 */
    256*1024 /* PMC-16AI64SS or SSA board */
};

/* number of channels for each board supported. */

int channels[] = {
	64, /* PMC-12AI64 */
	64, /* PMC-16AI64 */
	64  /* PMC-16AI64SS */
};

int main(int argc, char *argv[])
{
	struct device_register_params rw_q;
	unsigned long total;
	unsigned long buffer[BUFFER_SIZE];
	unsigned long count[MAX_CHANNELS];
	int board_type;
	int fd, res=0;
	int print;
	long i;
	unsigned long setting;
	int BuffersRead;
	int samples;
	int done;
	unsigned long errcode;
	int first = TRUE;

	printf("***** Build %s, %s *****\n",__DATE__,__TIME__);

	if (argc < 2) {
		printf("%s: missing argument\nusage: testapp [0|1|2|...]\n", argv[0]);
		return (1);
	}

	printf("about to open\n");
	strcat(driver_name, argv[1]);
	fd = open(driver_name, O_RDWR);

	if (fd < 0) {
		printf("%s: can not open device %s\n", argv[0], argv[1]);
		return (1);
	}

	// The device is now open. Any exit paths from now on must close
	// the driver.
	//
	//	init the board.

	res = ioctl(fd, (unsigned long)IOCTL_DEVICE_GET_DEVICE_TYPE, &board_type);
	if (res < 0) {
		printf("%s: ioctl IOCTL_SDI_GET_DEVICE_TYPE failed\n", argv[0]);
		goto OUT;
	}

	printf("open OK - board type: %s channels: %d\n", boards_supported[board_type],channels[board_type]);
	rw_q.DeviceRegister = 0x0;
	rw_q.ulValue = 0x2000;
	res = ioctl(fd, (unsigned long)IOCTL_DEVICE_READ_REGISTER, &rw_q);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_READ_REGISTER failed\n", argv[0]);
		goto OUT;
	}
    
    // tell the driver the size of the input buffer. 

    setting = hwBufferSize[board_type];
    res = ioctl(fd, IOCTL_DEVICE_SET_HW_BUFFER_SIZE, &setting);
    if (res < 0) {
        printf("%s: ioctl IOCTL_DEVICE_SET_HW_BUFFER_SIZE failed\n", argv[0]);
        goto OUT;
    }

	printf("before init: BCR is 0x%lx\n", rw_q.ulValue);
	res = ioctl(fd, IOCTL_DEVICE_INIT_BOARD, NULL);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_INIT_BOARD failed\n", argv[0]);
		goto OUT;
	}
	printf("board reset OK\n");
	rw_q.DeviceRegister = 0x0;
	rw_q.ulValue = 0x2000;
	res = ioctl(fd, (unsigned long)IOCTL_DEVICE_READ_REGISTER, &rw_q);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_READ_REGISTER failed\n", argv[0]);
		goto OUT;
	}

	printf("after init: BCR is 0x%lx\n", rw_q.ulValue);
	printf("autocalibration...\n");

	res = ioctl(fd, (unsigned long)IOCTL_DEVICE_START_AUTOCAL, NULL);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_START_AUTOCAL failed\n", argv[0]);
		goto OUT;
	}

	printf("auto calibration OK\n");

	// set timeout, in seconds
	setting = 10;
	res = ioctl(fd, IOCTL_DEVICE_SET_TIMEOUT, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_SET_TIMEOUT failed\n", argv[0]);
		goto OUT;
	}

	//	ensure that the acquisition hardware is stopped by selecting
	//  board control register clocking.

	res = ioctl(fd, IOCTL_DEVICE_GEN_DISABLE_A, NULL);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_GEN_DISABLE_A failed\n", argv[0]);
		goto OUT;
	}
	res = ioctl(fd, IOCTL_DEVICE_GEN_DISABLE_B, NULL);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_GEN_DISABLE_B failed\n", argv[0]);
		goto OUT;
	}
	
	// clear out any old data
	res = ioctl(fd, IOCTL_DEVICE_CLEAR_BUFFER, NULL);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_CLEAR_BUFFER failed\n", argv[0]);
		goto OUT;
	}

	//	set up input range 

	setting = INPUT_RANGE_10;
	res = ioctl(fd, IOCTL_DEVICE_INPUT_RANGE_CONFIG, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_INPUT_RANGE_CONFIG failed\n", argv[0]);
		goto OUT;
	}

	// number of channels to scan

	setting = SCAN_16_CHAN;
	res = ioctl(fd, IOCTL_DEVICE_SET_SCAN_SIZE, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_SET_SCAN_SIZE failed\n", argv[0]);
		goto OUT;
	}

	// divisor for rate generator "A."  Gen "B" is not used.

//	setting = 240;		// fast
	setting =1000;		// medium-ish
//	setting = 1600*4;	// slow
	res = ioctl(fd, IOCTL_DEVICE_SET_SCAN_RATE_A, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_SET_SCAN_RATE_A failed\n", argv[0]);
		goto OUT;
	}

	// set the input mode

//	setting = INPUT_ZERO_TEST;
	setting = INPUT_SINGLE_ENDED;
//	setting = INPUT_DIFFERENTIAL;
//	setting = INPUT_VREF_TEST;     

	res = ioctl(fd, IOCTL_DEVICE_INPUT_MODE_CONFIG, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_INPUT_MODE_CONFIG failed\n", argv[0]);
		goto OUT;
	}
	
	setting = DMA_DISABLE;
//	setting = DMA_ENABLE;
//	setting = DMA_DEMAND_MODE; // !! only for the SS boards !!
	res = ioctl(fd, IOCTL_DEVICE_ENABLE_DMA, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_ENABLE_DMA failed\n", argv[0]);
		goto OUT;
	}
//
// set the buffer threshold, the level where the driver sees the buffer ready condition.
//
	setting = 0x8000;
	res = ioctl(fd, IOCTL_DEVICE_SET_BUFFER_THRESHOLD, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_SET_BUFFER_THRESHOLD failed\n", argv[0]);
		goto OUT;
	}

    if (board_type==gsc16ai64ss)
    {
        setting = 0x2000;
        res = ioctl(fd, IOCTL_DEVICE_SET_BUFFER_THRESHOLD, &setting);
        if (res < 0) {
            printf("%s: ioctl IOCTL_DEVICE_SET_BUFFER_THRESHOLD failed\n", argv[0]);
            goto OUT;
        }

        setting = TRUE;
//        res = ioctl(fd, IOCTL_DEVICE_SET_THRESHOLD_4X_SSA, &setting);
        if (res < 0) {
            printf("%s: ioctl IOCTL_DEVICE_SET_THRESHOLD_4X_SSA failed\n", argv[0]);
            goto OUT;
        }
    }

	//
	// instruct driver to fail read if the buffer gets full (presumed data loss by buffer overrun)
	//
	setting = TRUE;
	res = ioctl(fd, IOCTL_DEVICE_DETECT_OVERFLOW, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_DETECT_OVERFLOW failed\n", argv[0]);
		goto OUT;
	}

//
// Always fill the user buffer completely.
//
	setting = TRUE;
	res = ioctl(fd, IOCTL_DEVICE_FILL_BUFFER, &setting);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_FILL_BUFFER failed\n", argv[0]);
		goto OUT;
	}

//
// start the hardware
//
	res = ioctl(fd, IOCTL_DEVICE_GEN_ENABLE_A, NULL);
	if (res < 0) {
		printf("%s: ioctl IOCTL_DEVICE_GEN_ENABLE_A failed\n", argv[0]);
		goto OUT;
	}
//
// set up for looping reading data.
//
//    goto OUT;
	for(i = 0; i < MAX_CHANNELS; i++) {
		count[i] = 0;
	}
	total = 0L;
	print = -1;
	printf("\n");
	BuffersRead=0;
	done = FALSE;

	init_keyboard();
	printf("\nPress any key to exit...\n");
	while (/*BuffersRead<1024&&*/ !done && !kbhit()) {
		int channel;
		memset(buffer,0x0,sizeof(buffer));
//		printf("\n");
		res = read(fd, buffer, BUFFER_SIZE_WORDS);
//		sleep(4); //comment in to force buffer overflow
		BuffersRead++;
		if (first)
		{
			first=FALSE;
			if (!(buffer[0]&CHANNEL_0_FLAG))
			{
				printf("\nSequence error - first channel not channel 0\n");
				goto OUTLOOP;
			}
		}

		if (res <= 0){
			printf("read error...res= %d\n",res);
			ioctl(fd, IOCTL_DEVICE_GET_DEVICE_ERROR, &errcode);
			printf("error code: %s\n",errorString[errcode]);

			goto OUTLOOP;
		}
		samples = res / sizeof(long);
		total += samples;
		channel=0;
		if (BuffersRead%100==0)
		{
			printf("%d \r",BuffersRead);
			fflush(stdout);
		}

//		printf("\nGot a buffer... size: %d bytes %d samples\n", res, samples);
		for(i = 0; (i < samples) && !done; i++) {
		//	printf(" [%d] %.8X ", i, buffer[i]);

			if (buffer[i]&CHANNEL_0_FLAG)
			{
				channel=0;
			}
			else
			{
				channel++;
			}
/*
			if (buffer[i]==0)
			{
				printf("Null data value! %d\n",channel);
				done = TRUE;
			}
*/
			if (channel<0||channel>MAX_CHANNELS) {
				printf("Illegal channel value! %d index: %d 0x%X\n",channel,i,i);
				done = TRUE;
			}
			count[channel]++;
		}
	}

	printf("Total Buffer read: %d\n",BuffersRead);
	for (i=0;i<MAX_CHANNELS;i++)
	{
		printf("%.8d ",count[i]);
		if ((i>0)&((i+1)%8==0))
			printf("\n");
	}
OUTLOOP:
	close_keyboard();
OUT:
	printf("\n");
	close(fd);
	return 0;
}
